__version__ = "0.5.3"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(Not currently on any branch)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = '97fe79e50a56b386554f47cc4433c4645495b72a'
__git_head_subject__ = 'Finished release v0.5.3.'
__git_head_message__ = '* release/v0.5.3:\n  Freezing dependencies\n  TRIVIAL --push-changes on freeze fix\n  TRIVIAL --push-changes on freeze fix\n  TRIVIAL --push-changes on freeze\n  Unfreezing dependencies\n  empty commit after version v0.5.2'
__git_dirty_diff__ = ''